#include "Multiplayer.h"
#include "RakPeerInterface.h"
#include "RakNetworkFactory.h"
#include "BitStream.h"
#include <stdlib.h> // For atoi
#include <cstring> // For strlen
#include <conio.h>
#include "RakNetStatistics.h"

#ifdef _COMPATIBILITY_1
#include "Compatibility1Includes.h" // Developers of a certain platform will know what to do here.
#elif defined(_WIN32)
#include <windows.h> // Sleep
#else
#include <unistd.h> // usleep
#include <cstdio>
#endif

bool quit;
bool sentPacket=false;

#define BIG_PACKET_SIZE 100000000

RakPeerInterface *peer1, *peer2;
char *text;

template <class InterfaceType>
class BigPacketMultiplayer : public Multiplayer<InterfaceType>
{
	void ProcessUnhandledPacket( Packet *packet, unsigned char packetIdentifier, InterfaceType *interfaceType )
	{
		RakNetStatisticsStruct *rss=peer2->GetStatistics(peer2->GetPlayerIDFromIndex(0));
		StatisticsToString(rss, text, 2);
		printf("%s", text);

		if (packet->data[0]==255)
		{
			bool dataValid=true;
			for (int i=1; i < BIG_PACKET_SIZE; i++)
			{
                if (packet->data[i]!=i%256)
				{
					dataValid=false;
					break;
				}
			}

			if (dataValid)
				printf("Test succeeded. %i bytes.\n", packet->length);
			else
				printf("Test failed. %i bytes.\n", packet->length);
		}
		else
			printf("Unknown packet %i: Test failed. %i bytes.\n", packet->data[0], packet->length);

		quit=true;
	}
};

int main(void)
{
	peer1=RakNetworkFactory::GetRakPeerInterface();;
	peer2=RakNetworkFactory::GetRakPeerInterface();;
	BigPacketMultiplayer<RakPeerInterface> peerMP1;
	Multiplayer<RakPeerInterface> peerMP2;

	text= new char [BIG_PACKET_SIZE];
	quit=false;
	RakNetTime nextStatTime = RakNet::GetTime() + 5000;

	peer1->SetMaximumIncomingConnections(1);
	peer1->Initialize(1, 60000, 0);
	peer2->Initialize(1, 60001, 0);
	peer2->Connect("127.0.0.1", 60000, 0, 0);
	peer2->SetMTUSize(1492);
	Sleep(50);
	//peer1->ApplyNetworkSimulator(26, 100, 50);
	//peer2->ApplyNetworkSimulator(26, 100, 50);

	RakNetTime start,stop;

	start=RakNet::GetTime();
	while (!quit)
	{
		peerMP1.ProcessPackets(peer1);
		peerMP2.ProcessPackets(peer2);

		if (sentPacket==false && peer2->GetPlayerIDFromIndex(0)!=UNASSIGNED_PLAYER_ID)
		{
			sentPacket=true;
			for (int i=0; i < BIG_PACKET_SIZE; i++)
				text[i]=i%256;
			text[0]=(char)255; // So it doesn't register as an internal ID
			peer2->Send(text, BIG_PACKET_SIZE, HIGH_PRIORITY, RELIABLE, 0, peer2->GetPlayerIDFromIndex(0), false);
		}

		if (RakNet::GetTime() > nextStatTime)
		{
			nextStatTime=RakNet::GetTime()+5000;
			RakNetStatisticsStruct *rss=peer2->GetStatistics(peer2->GetPlayerIDFromIndex(0));
			StatisticsToString(rss, text, 2);
			printf("%s", text);
		}

#ifdef _WIN32
		Sleep(30);
#else
		usleep(30 * 1000);
#endif
	}
	stop=RakNet::GetTime();
	printf("Test ran for %i ms. Press enter to quit\n", stop-start);
	gets(text);

	delete []text;
	RakNetworkFactory::DestroyRakPeerInterface(peer1);
	RakNetworkFactory::DestroyRakPeerInterface(peer2);

	return 0;
}
