#include "Multiplayer.h"
#include "RakServer.h"
#include "RakClient.h"
#include "RakNetworkFactory.h"
#include "BitStream.h"
#include "NetworkTypes.h"
using namespace RakNet;

// You can parse the input data in two ways.
// Either cast input to a struct (such as if you sent a struct)
// Or create a BitStream instance with input as data such as
// BitStream myBitStream(input, (numberOfBitsOfData-1)/8+1);
// where (numberOfBitsOfData-1)/8+1 is the number of bytes of data
// If you didn't pass any data then of course you don't do either
void serverRPC(RPCParameters *rpcParameters)
{
	if (rpcParameters->input)
		printf("In serverRPC Message = %s\n", rpcParameters->input);
	else
		printf("In serverRPC no input");

	// If the sender passed a bitstream pointer to the last parameter of RPC, then we can write to replyToSender as a return value.
	rpcParameters->replyToSender->Write("From serverRPC", (const int)strlen("From serverRPC")+1);
}

// You can parse the input data in two ways.
// Either cast input to a struct (such as if you sent a struct)
// Or create a BitStream instance with input as data such as
// BitStream myBitStream(input, (numberOfBitsOfData-1)/8+1);
// where (numberOfBitsOfData-1)/8+1 is the number of bytes of data
// If you didn't pass any data then of course you don't do either
void clientRPC1(RPCParameters *rpcParameters)
{
	if (rpcParameters->input)
		printf("In clientRPC1 Message = %s\n", rpcParameters->input);
	else
		printf("In clientRPC1 no input");

	// If the sender passed a bitstream pointer to the last parameter of RPC, then we can write to replyToSender as a return value.
	rpcParameters->replyToSender->Write("From clientRPC1", (const int)strlen("From clientRPC1")+1);
}

void clientRPC2(RPCParameters *rpcParameters)
{
	// We send the data as a bitstream.  However, we know the first 4 bytes is an unsigned int
	// That holds a timestamp.  Use the BitStream class to avoid 
	// cross-platform Endian problems in transmitting non-ASCII data.
	RakNet::BitStream tstamp(rpcParameters->input, sizeof(unsigned int), false);
	RakNetTime timestamp;
	tstamp.Read(timestamp);

	printf("In clientRPC2. Time sent: %i (%i diff). Message = %s\n", timestamp, RakNet::GetTime()-timestamp, rpcParameters->input+sizeof(unsigned int));
}

int main(void)
{
	Multiplayer<RakClientInterface> rakClientMultiplayer;
	Multiplayer<RakServerInterface> rakServerMultiplayer;

	bool isServer;
	int state=1;
	char str[256];
	RakNetTime nextActionTime;

	printf("Act as (s)erver or (c)lient?\n");
	gets(str);
	if (str[0]==0)
		return 1;

	if (str[0]=='s' || str[0]=='S')
	{
		isServer=true;
	}
	else
	{
		isServer=false;
	}

	RakClientInterface *rakClient;
	RakServerInterface *rakServer;

	if (isServer)
	{
		rakServer=RakNetworkFactory::GetRakServerInterface();
		REGISTER_STATIC_RPC(rakServer, serverRPC);
		rakServer->Start(32,0,30,2000);
	}
	else
	{
		rakClient=RakNetworkFactory::GetRakClientInterface();

		// You can register a function anytime
		// Note that although you are passing a function pointer, the name you are passing matters.  The macro uses
		// this name as a unique identifier.  Don't do *f = clientRPC and then register f
		REGISTER_STATIC_RPC(rakClient, clientRPC1);
		REGISTER_STATIC_RPC(rakClient, clientRPC2);
		rakClient->Connect("127.0.0.1", 2000, 2100, 0, 30);
	}	
		

	printf("RPC demo\n");

	nextActionTime=RakNet::GetTime()+1000;
	while (state<3)
	{
		if (isServer)
			rakServerMultiplayer.ProcessPackets(rakServer);
		else
			rakClientMultiplayer.ProcessPackets(rakClient);

		if (RakNet::GetTime() > nextActionTime)
		{
			if (state==1 && isServer==false)
			{
				// This shows the first form of RPC, which is just sending a char* and the number of bits
				// used.

				char str[200];
				printf("Enter any text string to show on the server:\n");
				gets(str);
				// Don't forget that RPC takes bits as the data length, not bytes!
				rakClient->RPC("serverRPC",str, (int)((strlen(str)+1)*8), HIGH_PRIORITY, RELIABLE, 0, false, UNASSIGNED_NETWORK_ID,0);
			}
			if (state==1 && isServer==true)
			{
				// This shows the second form of RPC, which is to send using a bitstream.  We pass
				// true to the last parameter of RPC, which means the first 4 bytes is a timestamp.
				// It will be modified to be system relative on the recipient
				BitStream bitStream;
				bitStream.Write(RakNet::GetTime()); // Write the system time to the first 4 bytes
				char str[200]; // Temporary buffer to hold a string
				printf("Enter any text string to show on the client:\n");
				gets(str);
				bitStream.Write(str, (int)strlen(str)+1); // Append the string to the bitstream
				rakServer->RPC("clientRPC1",str, (unsigned int)(strlen(str)+1)*8, HIGH_PRIORITY, RELIABLE, 0, UNASSIGNED_PLAYER_ID, true, false, UNASSIGNED_NETWORK_ID,0); // broadcast to everyone
				rakServer->RPC("clientRPC2",&bitStream, HIGH_PRIORITY, RELIABLE, 0, UNASSIGNED_PLAYER_ID, true, true, UNASSIGNED_NETWORK_ID,0); // broadcast to everyone
			}
			else if (state==2)
			{
				printf("Now testing simultaneous blocking RPC\n");
				printf("Press enter when you see this message in both console windows at the same time.\n");
				gets(str);
				RakNet::BitStream clientRPCReply;
				RakNet::BitStream serverRPCReply;
				if (isServer)
				{
					// Both systems are sending blocking RPCs to each other at the same time.  RakNet has code in place to prevent a deadlock because of this
					rakServer->RPC("clientRPC1","_ToClientString", (int)((strlen("_ToClientString")+1)*8), HIGH_PRIORITY, RELIABLE, 0, UNASSIGNED_PLAYER_ID, true, false, UNASSIGNED_NETWORK_ID, &serverRPCReply);
					printf("serverRPCReply=%s\n", serverRPCReply.GetData());
				}
				else
				{
					// Here we send a blocking RPC, which will not return until a reply is filled out to the bitstream passed as the last parameter
					rakClient->RPC("serverRPC","_ToServerString", (int)((strlen("_ToServerString")+1)*8), HIGH_PRIORITY, RELIABLE, 0, false, UNASSIGNED_NETWORK_ID, &clientRPCReply);
					// I can read out the data immediately - this is synchronous and is the benefit of using blocking RPC calls.
					printf("clientRPCReply=%s\n", clientRPCReply.GetData());
				}
			}

			nextActionTime = RakNet::GetTime() + 1000;
			state++;
		}
	}


	printf("Example complete.  Press enter to quit\n");
	gets(str);

	if (isServer)
	{	rakServer->Disconnect(0);


		UNREGISTER_STATIC_RPC(rakServer, serverRPC);
		RakNetworkFactory::DestroyRakServerInterface(rakServer);
	}
	else
	{
		rakClient->Disconnect(0);

		// This is not necessary since on shutdown everything is unregistered.  This is just here to show usage
		UNREGISTER_STATIC_RPC(rakClient, clientRPC1);
		UNREGISTER_STATIC_RPC(rakClient, clientRPC2);
		RakNetworkFactory::DestroyRakClientInterface(rakClient);
	}
	

	return 0;
}

